// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef DWAYLAND_VERSION_H
#define DWAYLAND_VERSION_H

#define DWAYLAND_VERSION_STRING "5.24.3"
#define DWAYLAND_VERSION_MAJOR 5
#define DWAYLAND_VERSION_MINOR 24
#define DWAYLAND_VERSION_PATCH 3
#define DWAYLAND_VERSION ((5<<16)|(24<<8)|(3))

#endif
